@@ -18,10 +18,10 @@ my %fields = (
                              TYPE SKUNUMBER/ ],
     HARDWARE         => [ qw/USERID OSVERSION PROCESSORN OSCOMMENTS CHECKSUM
                              PROCESSORT NAME PROCESSORS SWAP ETIME TYPE OSNAME
-                             IPADDR WORKGROUP DESCRIPTION MEMORY UUID VMID DNS
+                             IPADDR WORKGROUP DESCRIPTION MEMORY UUID DNS
                              LASTLOGGEDUSER USERDOMAIN DATELASTLOGGEDUSER
                              DEFAULTGATEWAY VMSYSTEM WINOWNER WINPRODID
-                             WINPRODKEY WINCOMPANY WINLANG CHASSIS_TYPE VMID
+                             WINPRODKEY WINCOMPANY WINLANG CHASSIS_TYPE
                              VMNAME VMHOSTSERIAL/ ],
     OPERATINGSYSTEM  => [ qw/KERNEL_NAME KERNEL_VERSION NAME VERSION FULL_NAME
                              SERVICE_PACK INSTALL_DATE FQDN DNS_DOMAIN
@@ -84,7 +84,7 @@ my %fields = (
                             CLASS SUBCLASS NAME/ ],
     USERS            => [ qw/LOGIN DOMAIN/ ],
     VIRTUALMACHINES  => [ qw/MEMORY NAME UUID STATUS SUBSYSTEM VMTYPE VCPU
-                             VMID MAC COMMENT OWNER SERIAL/ ],
+                             MAC COMMENT OWNER SERIAL/ ],
     VOLUME_GROUPS    => [ qw/VG_NAME PV_COUNT LV_COUNT ATTR SIZE FREE VG_UUID
                              VG_EXTENT_SIZE/ ],
 );
@@ -100,7 +100,7 @@ my %checks = (
         STATUS => qr/^(free|used)$/
     },
     NETWORKS => {
-        TYPE => qr/^(ethernet|wifi|aggregate|alias|dialup|loopback|bridge|fibrechannel)$/
+        TYPE => qr/^(ethernet|wifi|infiniband|aggregate|alias|dialup|loopback|bridge|fibrechannel)$/
     },
     CPUS => {
         ARCH => qr/^(MIPS|MIPS64|Alpha|SPARC|SPARC64|m68k|i386|x86_64|PowerPC|PowerPC64|ARM|AArch64)$/
@@ -141,6 +141,20 @@ sub getContent {
     return $self->{content};
 }
 
+sub getSection {
+    my ($self, $section) = @_;
+    ## no critic (ExplicitReturnUndef)
+    my $content = $self->getContent() or return undef;
+    return exists($content->{$section}) ? $content->{$section} : undef ;
+}
+
+sub getField {
+    my ($self, $section, $field) = @_;
+    ## no critic (ExplicitReturnUndef)
+    $section = $self->getSection($section) or return undef;
+    return exists($section->{$field}) ? $section->{$field} : undef ;
+}
+
 sub mergeContent {
     my ($self, $content) = @_;
 
@@ -274,6 +288,11 @@ sub computeLegacyValues {
     }
 }
 
+sub getHardware {
+    my ($self, $field) = @_;
+    return $self->getField('HARDWARE', $field);
+}
+
 sub setHardware {
     my ($self, $args) = @_;
 
@@ -306,6 +325,11 @@ sub setOperatingSystem {
     }
 }
 
+sub getBios {
+    my ($self, $field) = @_;
+    return $self->getField('BIOS', $field);
+}
+
 sub setBios {
     my ($self, $args) = @_;
 
@@ -476,6 +500,14 @@ an arbitrary label, used for server-side filtering
 
 Get content attribute.
 
+=head2 getSection($section)
+
+Get full machine inventory section.
+
+=head2 getField($section,$field)
+
+Get a field from a full machine inventory section.
+
 =head2 mergeContent($content)
 
 Merge content to the inventory.
@@ -505,6 +537,10 @@ ignore entry if already present
 
 Set inventory tag, an arbitrary label used for filtering on server side.
 
+=head2 getHardware($field)
+
+Get machine global information from known machine inventory.
+
 =head2 setHardware()
 
 Save global information regarding the machine.
@@ -513,6 +549,10 @@ Save global information regarding the machine.
 
 Operating System information.
 
+=head2 getBios($field)
+
+Get BIOS information from known inventory.
+
 =head2 setBios()
 
 Set BIOS information.
