@@ -8,7 +8,9 @@ use English qw(-no_match_vars);
 use FusionInventory::Agent::Tools;
 
 sub isEnabled {
-    canRun('lvs');
+    my (%params) = @_;
+    return 0 if $params{no_category}->{lvm};
+    return canRun('lvs');
 }
 
 sub doInventory {
