@@ -60,10 +60,7 @@ sub doInventory {
     my $inventory = $params{inventory};
     my $logger    = $params{logger};
 
-    # return immediatly if vm type has already been found
-    return if $inventory->{content}{HARDWARE}{VMSYSTEM} ne "Physical";
-
-    my $type = _getType($logger);
+    my $type = _getType($inventory->{content}->{BIOS}, $logger);
 
     # for consistency with HVM domU
     if (
@@ -76,22 +73,41 @@ sub doInventory {
         });
     }
 
-    my $vmid = $type eq 'Virtuozzo' ?
-        _getOpenVZVmID(logger => $logger) : undef;
-
-    my $uuid = $type eq 'Xen' ? _getXenUUID(logger => $logger) :
-               $type eq 'LXC' ? _getLibvirtLXC_UUID(logger => $logger) :
-               undef;
+    # compute a compound identifier, as Virtuozzo uses the same identifier
+    # for the host and for the guests
+    if ($type eq 'Virtuozzo') {
+        my $hostID  = $inventory->getHardware('UUID') || '';
+        my $guestID = getFirstMatch(
+            file => '/proc/self/status',
+            pattern => qr/^envID:\s*(\d+)/
+        ) || '';
+        $inventory->setHardware({ UUID => $hostID . '-' . $guestID });
+    }
 
     $inventory->setHardware({
         VMSYSTEM => $type,
-        UUID     => $uuid,
-        VMID     => $vmid
     });
 }
 
 sub _getType {
-    my ($logger) = @_;
+    my ($bios, $logger) = @_;
+
+    if ($bios->{SMANUFACTURER}) {
+        return 'Hyper-V' if $bios->{SMANUFACTURER} =~ /Microsoft/;
+        return 'VMware'  if $bios->{SMANUFACTURER} =~ /VMware/;
+    }
+    if ($bios->{BMANUFACTURER}) {
+        return 'QEMU'       if $bios->{BMANUFACTURER} =~ /(QEMU|Bochs)/;
+        return 'VirtualBox' if $bios->{BMANUFACTURER} =~ /(VirtualBox|innotek)/;
+        return 'Xen'        if $bios->{BMANUFACTURER} =~ /^Xen/;
+    }
+    if ($bios->{SMODEL}) {
+        return 'VMware'          if $bios->{SMODEL} =~ /VMware/;
+        return 'Virtual Machine' if $bios->{SMODEL} =~ /Virtual Machine/;
+    }
+    if ($bios->{BVERSION}) {
+        return 'VirtualBox'  if $bios->{BVERSION} =~ /VirtualBox/;
+    }
 
     if (-f '/.dockerinit') {
         return 'Docker';
@@ -226,37 +242,4 @@ sub _matchPatterns {
     }
 }
 
-sub _getOpenVZVmID {
-    return getFirstMatch(
-        file    => '/proc/self/status',
-        pattern => qr/^envID:\s*(\d+)/,
-        @_
-    );
-}
-
-sub _getXenUUID {
-    return getFirstLine(
-        file => '/sys/hypervisor/uuid',
-        @_
-    );
-}
-
-sub _getLibvirtLXC_UUID {
-    my (%params) = (
-        file => '/proc/1/environ',
-        @_
-    );
-
-    my @environ = split( '\0', getAllLines( %params ) );
-
-    foreach my $variable (@environ) {
-        next unless $variable =~ /^LIBVIRT_LXC_UUID/;
-        my (undef, $value) = split('=', $variable);
-        return $value;
-    }
-
-    ## no critic (ExplicitReturnUndef)
-    return undef;
-}
-
 1;
