@@ -66,6 +66,7 @@ sub getDmidecodeInfos {
             $2 eq 'N/A'                        ||
             $2 eq 'Not Specified'              ||
             $2 eq 'Not Present'                ||
+            $2 eq 'Unknown'                    ||
             $2 eq '<BAD INDEX>'                ||
             $2 eq '<OUT OF SPEC>'              ||
             $2 eq '<OUT OF SPEC><OUT OF SPEC>' ;
@@ -87,24 +88,26 @@ sub getCpusFromDmidecode {
 
     my @cpus;
     foreach my $info (@{$infos->{4}}) {
-        next if $info->{Status} && $info->{Status} =~ /Unpopulated/i;
+        next if $info->{Status} && $info->{Status} =~ /Unpopulated|Disabled/i;
 
-        my $proc_manufacturer = $info->{'Processor Manufacturer'};
-        my $proc_version      = $info->{'Processor Version'};
+        my $manufacturer = $info->{'Manufacturer'} ||
+                           $info->{'Processor Manufacturer'};
+        my $version      = $info->{'Version'} ||
+                           $info->{'Processor Version'};
 
         # VMware
         next if
-            ($proc_manufacturer && $proc_manufacturer eq '000000000000') &&
-            ($proc_version      && $proc_version eq '00000000000000000000000000000000');
+            ($manufacturer && $manufacturer eq '000000000000') &&
+            ($version      && $version eq '00000000000000000000000000000000');
 
         my $cpu = {
-            SERIAL     => $info->{'Serial Number'},
-            ID         => $info->{ID},
-            CORE       => $info->{'Core Count'} || $info->{'Core Enabled'},
-            THREAD     => $info->{'Thread Count'},
-            FAMILYNAME => $info->{'Family'}
+            SERIAL       => $info->{'Serial Number'},
+            ID           => $info->{ID},
+            CORE         => $info->{'Core Count'} || $info->{'Core Enabled'},
+            THREAD       => $info->{'Thread Count'},
+            FAMILYNAME   => $info->{'Family'},
+            MANUFACTURER => $manufacturer
         };
-        $cpu->{MANUFACTURER} = $info->{'Manufacturer'} || $info->{'Processor Manufacturer'};
         $cpu->{NAME} =
             ($cpu->{MANUFACTURER} =~ /Intel/ ? $info->{'Family'} : undef) ||
             $info->{'Version'}                                     ||
@@ -129,15 +132,7 @@ sub getCpusFromDmidecode {
                 $cpu->{SPEED} = $1 * 1000;
             }
         }
-        if (!$cpu->{SPEED} && $info->{'Max Speed'}) {
-            # We only look for 3 digit Mhz frequency to avoid abvious bad
-            # value like 30000 (#633)
-            if ($info->{'Max Speed'} =~ /^\s*(\d{3,4})\s*Mhz/i) {
-                $cpu->{SPEED} = $1;
-            } elsif ($info->{'Max Speed'} =~ /^\s*(\d+)\s*Ghz/i) {
-                $cpu->{SPEED} = $1 * 1000;
-            }
-        }
+
         if (!$cpu->{SPEED} && $info->{'Current Speed'}) {
             if ($info->{'Current Speed'} =~ /^\s*(\d{3,4})\s*Mhz/i) {
                 $cpu->{SPEED} = $1;
