@@ -5,7 +5,6 @@ use warnings;
 use base 'Exporter';
 
 use English qw(-no_match_vars);
-use List::Util qw(first);
 
 use FusionInventory::Agent::Tools;
 use FusionInventory::Agent::Tools::Network;
@@ -221,19 +220,42 @@ my %consumable_types = (
 # printer-specific page counter variables
 my %printer_pagecounters_variables = (
     TOTAL      => {
-        oid   => '.1.3.6.1.2.1.43.10.2.1.4.1.1'
+        oid   => [
+            '.1.3.6.1.4.1.1347.42.2.1.1.1.6.1.1', #Kyocera specific counter
+            '.1.3.6.1.2.1.43.10.2.1.4.1.1'        #Default Value
+            ]
+    },
+    BLACK      => {
+        oid   => '.1.3.6.1.4.1.1347.42.2.1.1.1.7.1.1' #Kyocera specific counter
+    },
+    COLOR      => {
+        oid   => '.1.3.6.1.4.1.1347.42.2.1.1.1.8.1.1' #Kyocera specific counter
     },
-    BLACK      => { },
-    COLOR      => { },
     RECTOVERSO => { },
-    SCANNED    => { },
-    PRINTTOTAL => { },
-    PRINTBLACK => { },
-    PRINTCOLOR => { },
-    COPYTOTAL  => { },
-    COPYBLACK  => { },
-    COPYCOLOR  => { },
-    FAXTOTAL   => { },
+    SCANNED    => {
+        oid   => '.1.3.6.1.4.1.1347.46.10.1.1.5.3' #Kyocera specific counter ( total scan counter)
+    },
+    PRINTTOTAL => {
+        oid   => '.1.3.6.1.4.1.1347.42.3.1.1.1.1.2' #Kyocera specific counter
+    },
+    PRINTBLACK => {
+        oid   => '.1.3.6.1.4.1.1347.42.3.1.2.1.1.1.1' #Kyocera specific counter
+    },
+    PRINTCOLOR => {
+        oid   => '.1.3.6.1.4.1.1347.42.3.1.2.1.1.1.2' #Kyocera specific counter
+    },
+    COPYTOTAL  => {
+        oid   => '.1.3.6.1.4.1.1347.42.3.1.1.1.1.2' #Kyocera specific counter
+    },
+    COPYBLACK  => {
+        oid   => '.1.3.6.1.4.1.1347.42.3.1.2.1.1.2.1' #Kyocera specific counter
+    },
+    COPYCOLOR  => {
+        oid   => '.1.3.6.1.4.1.1347.42.3.1.2.1.1.2.2' #Kyocera specific counter
+    },
+    FAXTOTAL   => {
+        oid   => '.1.3.6.1.4.1.1347.42.3.1.1.1.1.4'  #Kyocera specific counter
+    }
 );
 
 sub getDeviceInfo {
@@ -367,7 +389,7 @@ sub _getSysObjectIDInfo {
         \.1\.3\.6\.1\.4\.1
     )/x;
     my ($manufacturer_id, $device_id) =
-        $params{id} =~ /^ $prefix \. (\d+) \. ([\d.]+) $/x;
+        $params{id} =~ /^ $prefix \. (\d+) (?:\. ([\d.]+))? $/x;
 
     if (!$manufacturer_id) {
         $logger->debug("invalid sysobjectID $params{id}: no manufacturer ID")
@@ -378,18 +400,19 @@ sub _getSysObjectIDInfo {
     if (!$device_id) {
         $logger->debug("invalid sysobjectID $params{id}: no device ID")
             if $logger;
-        return;
     }
 
     my $match;
 
     # attempt full match first
-    $match = $sysobjectid{$manufacturer_id . '.' . $device_id};
-    if ($match) {
-        $logger->debug(
-            "full match for sysobjectID $params{id} in database"
-        ) if $logger;
-        return $match;
+    if ($device_id) {
+        $match = $sysobjectid{$manufacturer_id . '.' . $device_id};
+        if ($match) {
+            $logger->debug(
+                "full match for sysobjectID $params{id} in database"
+            ) if $logger;
+            return $match;
+        }
     }
 
     # fallback to partial match
@@ -436,28 +459,29 @@ sub _loadSysObjectIDDatabase {
 sub _getSerial {
     my ($snmp, $type) = @_;
 
-    my @network_oids = (
-        '.1.3.6.1.2.1.47.1.1.1.1.11.1',    # Entity-MIB::entPhysicalSerialNum
-        '.1.3.6.1.2.1.47.1.1.1.1.11.2',    # Entity-MIB::entPhysicalSerialNum
-        '.1.3.6.1.2.1.47.1.1.1.1.11.1001', # Entity-MIB::entPhysicalSerialNum
-        '.1.3.6.1.4.1.2636.3.1.3.0',       # Juniper-MIB
-        '.1.3.6.1.4.1.248.14.1.1.9.1.10.1',# Hirschman MIB
-    );
+    # Entity-MIB::entPhysicalSerialNum
+    my $entPhysicalSerialNum = $snmp->get_first('.1.3.6.1.2.1.47.1.1.1.1.11');
+    return _getCanonicalSerialNumber($entPhysicalSerialNum)
+        if $entPhysicalSerialNum;
 
-    my @printer_oids = (
-        '.1.3.6.1.2.1.43.5.1.1.17.1',      # Printer-MIB::prtGeneralSerialNumber
+    # Printer-MIB::prtGeneralSerialNumber
+    my $prtGeneralSerialNumber = $snmp->get_first('.1.3.6.1.2.1.43.5.1.1.17');
+    return _getCanonicalSerialNumber($prtGeneralSerialNumber)
+        if $prtGeneralSerialNumber;
+
+    # vendor specific OIDs
+    my @oids = (
+        '.1.3.6.1.4.1.2636.3.1.3.0',             # Juniper-MIB
+        '.1.3.6.1.4.1.248.14.1.1.9.1.10.1',      # Hirschman MIB
         '.1.3.6.1.4.1.253.8.53.3.2.1.3.1',       # Xerox-MIB
         '.1.3.6.1.4.1.367.3.2.1.2.1.4.0',        # Ricoh-MIB
         '.1.3.6.1.4.1.641.2.1.2.1.6.1',          # Lexmark-MIB
         '.1.3.6.1.4.1.1602.1.2.1.4.0',           # Canon-MIB
         '.1.3.6.1.4.1.2435.2.3.9.4.2.1.5.5.1.0', # Brother-MIB
+        '.1.3.6.1.4.1.318.1.1.4.1.5.0',          # MasterSwitch-MIB
+        '.1.3.6.1.4.1.6027.3.8.1.1.5.0',         # F10-C-SERIES-CHASSIS-MIB
+        '.1.3.6.1.4.1.6027.3.10.1.2.2.1.12.1',   # FORCE10-SMI
     );
-
-    my @oids =
-        $type && $type eq 'NETWORKING' ? @network_oids :
-        $type && $type eq 'PRINTER'    ? @printer_oids :
-                                         (@network_oids, @printer_oids);
-
     foreach my $oid (@oids) {
         my $value = $snmp->get($oid);
         next unless $value;
@@ -470,30 +494,16 @@ sub _getSerial {
 sub _getFirmware {
     my ($snmp, $type) = @_;
 
-    my $entPhysicalSoftwareRev = $snmp->walk('.1.3.6.1.2.1.47.1.1.1.1.10');
-    if ($entPhysicalSoftwareRev) {
-        my $firmware =
-            first { $_ }
-            map   { $entPhysicalSoftwareRev->{$_} }
-            sort  { $a <=> $b }
-            keys %$entPhysicalSoftwareRev;
-        return $firmware if $firmware;
-    }
+    my $entPhysicalSoftwareRev = $snmp->get_first('.1.3.6.1.2.1.47.1.1.1.1.10');
+    return $entPhysicalSoftwareRev if $entPhysicalSoftwareRev;
 
-    my $entPhysicalFirmwareRev = $snmp->walk('.1.3.6.1.2.1.47.1.1.1.1.9');
-    if ($entPhysicalFirmwareRev) {
-        my $firmware =
-            first { $_ }
-            map   { $entPhysicalFirmwareRev->{$_} }
-            sort  { $a <=> $b }
-            keys %$entPhysicalFirmwareRev;
-        return $firmware if $firmware;
-    }
+    my $entPhysicalFirmwareRev = $snmp->get_first('.1.3.6.1.2.1.47.1.1.1.1.9');
+    return $entPhysicalFirmwareRev if $entPhysicalFirmwareRev;
 
-    my $ios_version = $snmp->walk('.1.3.6.1.4.1.9.9.25.1.1.1.2.5');
+    my $ios_version = $snmp->get('.1.3.6.1.4.1.9.9.25.1.1.1.2.5');
     return $ios_version if $ios_version;
 
-    my $firmware = $snmp->walk('.1.3.6.1.4.1.248.14.1.1.2.0');
+    my $firmware = $snmp->get('.1.3.6.1.4.1.248.14.1.1.2.0');
     return $firmware if $firmware;
 
     return;
@@ -769,8 +779,16 @@ sub _setPrinterProperties {
     # page counters
     foreach my $key (keys %printer_pagecounters_variables) {
         my $variable = $printer_pagecounters_variables{$key};
-        my $oid = $variable->{oid};
-        my $value = $snmp->get($oid);
+        my $value;
+        if (ref $variable->{oid} eq 'ARRAY') {
+            foreach my $oid (@{$variable->{oid}}) {
+                $value = $snmp->get($oid);
+                last if $value;
+            }
+        } else {
+            my $oid = $variable->{oid};
+            $value = $snmp->get($oid);
+        }
         next unless defined $value;
         if (!_isInteger($value)) {
             $logger->error("incorrect counter value $value, check $variable->{mapping} mapping") if $logger;
@@ -1415,15 +1433,29 @@ sub _getVlans {
     # each result matches either of the following schemes:
     # $prefix.$i.$j = $value, with $j as port id, and $value as vlan id
     # $prefix.$i    = $value, with $i as port id, and $value as vlan id
-    foreach my $suffix (sort keys %{$vmPortStatus}) {
-        my $port_id = _getElement($suffix, -1);
-        my $vlan_id = $vmPortStatus->{$suffix};
-        my $name    = $vtpVlanName->{$vlan_id};
+    # work with Cisco and Juniper switches
+    if($vtpVlanName and $vmPortStatus){
+        foreach my $suffix (sort keys %{$vmPortStatus}) {
+            my $port_id = _getElement($suffix, -1);
+            my $vlan_id = $vmPortStatus->{$suffix};
+            my $name    = $vtpVlanName->{$vlan_id};
 
-        push @{$results->{$port_id}}, {
-            NUMBER => $vlan_id,
-            NAME   => $name
-        };
+            push @{$results->{$port_id}}, {
+                NUMBER => $vlan_id,
+                NAME   => $name
+            };
+        }
+    }
+
+    # For other switches, we use another method
+    my $vlanId = $snmp->walk('.1.0.8802.1.1.2.1.5.32962.1.2.1.1.1');
+    if($vlanId){
+        while (my ($port, $vlan) = each %{$vlanId}) {
+            push @{$results->{$port}}, {
+                NUMBER => $vlan,
+                NAME   => "VLAN " . $vlan
+            };
+        }
     }
 
     return $results;
